// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:dartlings/range.dart';

import 'package:darmatch/matchers.dart';
import 'package:detester/console.dart';
part 'package:detester/console_test.dart';

suite() => [
  testRangeFunction,
  testRangeConstructor,
  testRangeIterable
];

testRangeFunction() {
  var r = range(0, 10);
  expectThat(r, iterableLength(equals(10)));
  expectThat(r, orderedEquals([0, 1, 2, 3, 4, 5, 6, 7, 8, 9]));
  expectThat(r.contains(-1), isFalse());
  expectThat(r.contains(0), isTrue());
  expectThat(r.contains(5), isTrue());
  expectThat(r.contains(9), isTrue());
  expectThat(r.contains(10), isFalse());

  r = range(0, 10, inclusive: true);
  expectThat(r, iterableLength(equals(11)));
  expectThat(r, orderedEquals([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]));

  expectThat(() => range(0, 2), returnsNormally());
  expectThat(() => range(0, 1), returnsNormally());
  expectThat(() => range(0, 0), throws());
  expectThat(() => range(0, -1), throws());
  expectThat(() => range(0, 2, inclusive: true), returnsNormally());
  expectThat(() => range(0, 1, inclusive: true), returnsNormally());
  expectThat(() => range(0, 0, inclusive: true), returnsNormally());
  expectThat(() => range(0, -1, inclusive: true), throws());
}

testRangeConstructor() {
  var r = new Range(0, 10);
  expectThat(r, iterableLength(equals(11)));
  expectThat(r, orderedEquals([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]));

  r = new Range(0, 10, step: 2);
  expectThat(r, iterableLength(equals(6)));
  expectThat(r, orderedEquals([0, 2, 4, 6, 8, 10]));

  r = new Range(0, 10, step: 3);
  expectThat(r, iterableLength(equals(4)));
  expectThat(r, orderedEquals([0, 3, 6, 9]));

  r = new Range(0, 10, step: 4);
  expectThat(r, iterableLength(equals(3)));
  expectThat(r, orderedEquals([0, 4, 8]));

  r = new Range(0, 10, step: 5);
  expectThat(r, iterableLength(equals(3)));
  expectThat(r, orderedEquals([0, 5, 10]));

  r = new Range(0, 10, step: 6);
  expectThat(r, iterableLength(equals(2)));
  expectThat(r, orderedEquals([0, 6]));

  r = new Range(0, 10, step: 10);
  expectThat(r, iterableLength(equals(2)));
  expectThat(r, orderedEquals([0, 10]));

  r = new Range(0, 10, step: 11);
  expectThat(r, iterableLength(equals(1)));
  expectThat(r, orderedEquals([0]));

  r = new Range(10, 0);
  expectThat(r, iterableLength(equals(11)));
  expectThat(r, orderedEquals([10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0]));

  r = new Range(10, 0, step: -2);
  expectThat(r, iterableLength(equals(6)));
  expectThat(r, orderedEquals([10, 8, 6, 4, 2, 0]));

  r = new Range(10, 0, step: -3);
  expectThat(r, iterableLength(equals(4)));
  expectThat(r, orderedEquals([10, 7, 4, 1]));

  r = new Range(10, 0, step: -4);
  expectThat(r, iterableLength(equals(3)));
  expectThat(r, orderedEquals([10, 6, 2]));

  r = new Range(10, 0, step: -5);
  expectThat(r, iterableLength(equals(3)));
  expectThat(r, orderedEquals([10, 5, 0]));

  r = new Range(10, 0, step: -6);
  expectThat(r, iterableLength(equals(2)));
  expectThat(r, orderedEquals([10, 4]));

  r = new Range(10, 0, step: -10);
  expectThat(r, iterableLength(equals(2)));
  expectThat(r, orderedEquals([10, 0]));

  r = new Range(10, 0, step: -11);
  expectThat(r, iterableLength(equals(1)));
  expectThat(r, orderedEquals([10]));

  expectThat(() => new Range(0, 1), returnsNormally());
  expectThat(() => new Range(0, 1, step: 1), returnsNormally());
  expectThat(() => new Range(0, 1, step: 0), throws());
  expectThat(() => new Range(0, 1, step: -1), throws());

  expectThat(() => new Range(0, 0), returnsNormally());
  expectThat(() => new Range(0, 0, step: 1), returnsNormally());
  expectThat(() => new Range(0, 0, step: 0), returnsNormally());
  expectThat(() => new Range(0, 0, step: -1), returnsNormally());

  expectThat(() => new Range(0, -1), returnsNormally());
  expectThat(() => new Range(0, -1, step: 1), throws());
  expectThat(() => new Range(0, -1, step: 0), throws());
  expectThat(() => new Range(0, -1, step: -1), returnsNormally());
}

testRangeIterable() {
  var r = range(0, 5);

  expectThat(r.contains(-1), isFalse());
  expectThat(r.contains(0), isTrue());
  expectThat(r.contains(1), isTrue());
  expectThat(r.contains(2), isTrue());
  expectThat(r.contains(3), isTrue());
  expectThat(r.contains(4), isTrue());
  expectThat(r.contains(5), isFalse());
  expectThat(r.iterator, isNotNull());
  expectThat(r.length, equals(5));
  expectThat(r.isEmpty, isFalse());

  expectThat(r.map((e) => e * 2), orderedEquals([0, 2, 4, 6, 8]));
  expectThat(r.where((e) => e < 3), orderedEquals([0, 1, 2]));
  expectThat(r.expand((e) => [e, e + 1]), orderedEquals([0, 1, 1, 2, 2, 3, 3, 4, 4, 5]));
  r.forEach((e) => expectThat(e, isNotNull()));
  expectThat(r.reduce(0, (accum, e) => accum + e), equals(10));
  expectThat(r.every((e) => e < 10), isTrue());
  expectThat(r.join(', '), equals('0, 1, 2, 3, 4'));
  expectThat(r.any((e) => e < 3), isTrue());
  expectThat(r.toList(), new IsType<List>());
  expectThat(r.toSet(), new IsType<Set>());
  expectThat(r.min(), equals(0));
  expectThat(r.max(), equals(4));
  expectThat(r.take(2), orderedEquals([0, 1]));
  expectThat(r.takeWhile((e) => e < 2), orderedEquals([0, 1]));
  expectThat(r.skip(2), orderedEquals([2, 3, 4]));
  expectThat(r.skipWhile((e) => e < 2), orderedEquals([2, 3, 4]));
  expectThat(r.first, equals(0));
  expectThat(r.last, equals(4));
  expectThat(() => r.single, throws());
  expectThat(new Range(1, 1).first, equals(1));
  expectThat(r.firstWhere((e) => e % 2 != 0), equals(1));
  expectThat(r.lastWhere((e) => e % 2 != 0), equals(3));
  expectThat(() => r.singleWhere((e) => e % 2 != 0), throws());
  expectThat(new Range(1, 1).singleWhere((e) => e % 2 != 0), equals(1));
  expectThat(r.elementAt(2), equals(2));
}
